﻿using System;
using System.Collections.Generic;

namespace TestProject
{
    /// <summary>
    /// 스레드 안전 정렬 리스트
    /// </summary>
    /// <typeparam name="TItemKey">항목 키 타입</typeparam>
    /// <typeparam name="TItem">항목 타입</typeparam>
    public class ThreadSafeSortedList<TItemKey, TItem> : SortedList<TItemKey, TItem>
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region Field

        /// <summary>
        /// 동기 객체
        /// </summary>
        protected object syncObject;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 카운트 - Count

        /// <summary>
        /// 카운트
        /// </summary>
        public new int Count
        {
            get
            {
                lock(this.syncObject)
                {
                    return base.Count;
                }
            }
        }

        #endregion
        #region 인덱서 - this[itemKey]

        /// <summary>
        /// 인덱서
        /// </summary>
        /// <param name="itemKey">항목 키</param>
        /// <returns>항목</returns>
        public new TItem this[TItemKey itemKey]
        {
            get
            {
                lock(this.syncObject)
                {
                    return base[itemKey];
                }
            }
            set
            {
                lock(this.syncObject)
                {
                    base[itemKey] = value;
                }
            }
        }

        #endregion
        #region 인덱서 - this[itemKeyIndex]

        /// <summary>
        /// 인덱서
        /// </summary>
        /// <param name="itemKeyIndex">항목 키 인덱스</param>
        /// <returns>항목</returns>
        public TItem this[int itemKeyIndex]
        {
            get
            {
                lock(this.syncObject)
                {
                    TItemKey itemKey = base.Keys[itemKeyIndex];

                    return base[itemKey];
                }
            }
            set
            {
                lock(this.syncObject)
                {
                    TItemKey itemKey = base.Keys[itemKeyIndex];

                    base[itemKey] = value;
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ThreadSafeSortedList()

        /// <summary>
        /// 생성자
        /// </summary>
        public ThreadSafeSortedList()
        {
            this.syncObject = new Object();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 항목 키 포함 여부 구하기 - ContainsItemKey(itemKey)

        /// <summary>
        /// 항목 키 포함 여부 구하기
        /// </summary>
        /// <param name="itemKey">항목 키</param>
        /// <returns>항목 키 포함 여부</returns>
        public bool ContainsItemKey(TItemKey itemKey)
        {
            lock(this.syncObject)
            {
                return base.ContainsKey(itemKey);
            }
        }

        #endregion
        #region 항목 추가하기 - AddItem(itemKey, item)

        /// <summary>
        /// 항목 추가하기
        /// </summary>
        /// <param name="itemKey">항목 키</param>
        /// <param name="item">항목</param>
        /// <returns>처리 결과</returns>
        public bool AddItem(TItemKey itemKey, TItem item)
        {
            lock(this.syncObject)
            {
                if(base.ContainsKey(itemKey))
                {
                    return false;
                }
                else
                {
                    base.Add(itemKey, item);

                    return true;
                }
            }
        }

        #endregion
        #region 항목 제거하기 - RemoveItemAt(index)

        /// <summary>
        /// 항목 제거하기
        /// </summary>
        /// <param name="index">인덱스</param>
        public void RemoveItemAt(int index)
        {
            lock(this.syncObject)
            {
                RemoveAt(index);
            }
        }

        #endregion
        #region 항목 제거하기 - RemoveItem(itemKey)

        /// <summary>
        /// 항목 제거하기
        /// </summary>
        /// <param name="itemKey">항목 키</param>
        /// <returns>처리 결과</returns>
        public bool RemoveItem(TItemKey itemKey)
        {
            lock(this.syncObject)
            {
                return base.Remove(itemKey);
            }
        }

        #endregion
        #region 데이터 제거하기 - ClearData()

        /// <summary>
        /// 데이터 제거하기
        /// </summary>
        public void ClearData()
        {
            lock(this.syncObject)
            {
                Clear();
            }
        }

        #endregion
    }
}