﻿namespace TestProject
{
    /// <summary>
    /// 스레드 안전 정렬 리스트 큐
    /// </summary>
    /// <typeparam name="TItemKey">항목 키 타입</typeparam>
    /// <typeparam name="TItem">항목 타입</typeparam>
    public class ThreadSafeSortedListQueue<TItemKey, TItem> : ThreadSafeSortedList<TItemKey, TItem>
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 항목 넣기 - EnqueueItem(itemKey, item)

        /// <summary>
        /// 항목 넣기
        /// </summary>
        /// <param name="itemKey">항목 키</param>
        /// <param name="item">항목</param>
        /// <returns>처리 결과</returns>
        public bool EnqueueItem(TItemKey itemKey, TItem item)
        {
            return AddItem(itemKey, item);
        }

        #endregion
        #region 항목 엿보기 - PeekItem()

        /// <summary>
        /// 항목 엿보기
        /// </summary>
        /// <returns>항목</returns>
        public TItem PeekItem()
        {
            lock(this.syncObject)
            {
                if(base.Count > 0)
                {
                    return this[0];
                }
                else
                {
                    return default(TItem);
                }
            }
        }

        #endregion
        #region 항목 꺼내기 - DequeueItem()

        /// <summary>
        /// 항목 꺼내기
        /// </summary>
        /// <returns>항목</returns>
        public TItem DequeueItem()
        {
            lock(this.syncObject)
            {
                if(Count > 0)
                {
                    TItem item = this[0];

                    base.RemoveAt(0);

                    return item;
                }
                else
                {
                    return default(TItem);
                }
            }
        }

        #endregion
    }
}